@@ -365,7 +365,7 @@ func parseCommitSpec(dEnv *env.DoltEnv, apr *argparser.ArgParseResults) (*doltdb
 }
 
 func execShell(sqlCtx *sql.Context, readOnly bool, mrEnv env.MultiRepoEnv, roots map[string]*doltdb.RootValue, format resultFormat) errhand.VerboseError {
-	dbs := CollectDBs(mrEnv, newDatabase)
+	dbs := CollectDBs(mrEnv)
 	se, err := newSqlEngine(sqlCtx, readOnly, mrEnv, roots, format, dbs...)
 	if err != nil {
 		return errhand.VerboseErrorFromError(err)
@@ -379,32 +379,44 @@ func execShell(sqlCtx *sql.Context, readOnly bool, mrEnv env.MultiRepoEnv, roots
 }
 
 func execBatch(sqlCtx *sql.Context, readOnly bool, mrEnv env.MultiRepoEnv, roots map[string]*doltdb.RootValue, batchInput io.Reader, format resultFormat) errhand.VerboseError {
-	dbs := CollectDBs(mrEnv, newBatchedDatabase)
+	dbs := CollectDBs(mrEnv)
 	se, err := newSqlEngine(sqlCtx, readOnly, mrEnv, roots, format, dbs...)
 	if err != nil {
 		return errhand.VerboseErrorFromError(err)
 	}
 
+	// In batch mode, we need to set a couple flags on the session to prevent flushes to disk after every commit
+	dsqle.DSessFromSess(sqlCtx.Session).EnableBatchedMode()
+	err = sqlCtx.Session.SetSessionVariable(sqlCtx, sql.AutoCommitSessionVar, true)
+	if err != nil {
+		return errhand.VerboseErrorFromError(err)
+	}
+
 	err = runBatchMode(sqlCtx, se, batchInput)
 	if err != nil {
+		// If we encounter an error, flush what we have so far to disk before exiting, except in the case of merge
+		// errors, which have already updated the repo state all they're going to (and writing session root on top of
+		// them would overwrite these changes)
+		// TODO: this is a mess, merge conflicts need to follow the same code path as everything else
+		if err == doltdb.ErrUnresolvedConflicts || err == doltdb.ErrMergeActive {
+			return errhand.BuildDError("Error processing batch").Build()
+		}
+
+		_ = flushBatchedEdits(sqlCtx, se)
+		_ = writeRoots(sqlCtx, se, mrEnv, roots)
+
 		return errhand.BuildDError("Error processing batch").Build()
 	}
 
 	return writeRoots(sqlCtx, se, mrEnv, roots)
 }
 
-type createDBFunc func(name string, dEnv *env.DoltEnv) dsqle.Database
-
 func newDatabase(name string, dEnv *env.DoltEnv) dsqle.Database {
 	return dsqle.NewDatabase(name, dEnv.DbData())
 }
 
-func newBatchedDatabase(name string, dEnv *env.DoltEnv) dsqle.Database {
-	return dsqle.NewBatchedDatabase(name, dEnv.DbData())
-}
-
 func execQuery(sqlCtx *sql.Context, readOnly bool, mrEnv env.MultiRepoEnv, roots map[string]*doltdb.RootValue, query string, format resultFormat) errhand.VerboseError {
-	dbs := CollectDBs(mrEnv, newDatabase)
+	dbs := CollectDBs(mrEnv)
 	se, err := newSqlEngine(sqlCtx, readOnly, mrEnv, roots, format, dbs...)
 	if err != nil {
 		return errhand.VerboseErrorFromError(err)
@@ -427,10 +439,10 @@ func execQuery(sqlCtx *sql.Context, readOnly bool, mrEnv env.MultiRepoEnv, roots
 
 // CollectDBs takes a MultiRepoEnv and creates Database objects from each environment and returns a slice of these
 // objects.
-func CollectDBs(mrEnv env.MultiRepoEnv, createDB createDBFunc) []dsqle.Database {
+func CollectDBs(mrEnv env.MultiRepoEnv) []dsqle.Database {
 	dbs := make([]dsqle.Database, 0, len(mrEnv))
 	_ = mrEnv.Iter(func(name string, dEnv *env.DoltEnv) (stop bool, err error) {
-		db := createDB(name, dEnv)
+		db := newDatabase(name, dEnv)
 		dbs = append(dbs, db)
 		return false, nil
 	})
@@ -1095,7 +1107,7 @@ func processNonBatchableQuery(ctx *sql.Context, se *sqlEngine, query string, sql
 	if rowIter != nil {
 		err = mergeResultIntoStats(sqlStatement, rowIter, batchEditStats)
 		if err != nil {
-			return fmt.Errorf("error executing statement: %v", err.Error())
+			return err
 		}
 
 		// Some statement types should print results, even in batch mode.
@@ -1125,7 +1137,7 @@ func processNonBatchableQuery(ctx *sql.Context, se *sqlEngine, query string, sql
 func processBatchableEditQuery(ctx *sql.Context, se *sqlEngine, query string, sqlStatement sqlparser.Statement) (returnErr error) {
 	_, rowIter, err := se.query(ctx, query)
 	if err != nil {
-		return fmt.Errorf("Error inserting rows: %v", err.Error())
+		return err
 	}
 
 	if rowIter != nil {
@@ -1137,7 +1149,7 @@ func processBatchableEditQuery(ctx *sql.Context, se *sqlEngine, query string, sq
 		}()
 		err = mergeResultIntoStats(sqlStatement, rowIter, batchEditStats)
 		if err != nil {
-			return fmt.Errorf("Error inserting rows: %v", err.Error())
+			return err
 		}
 	}
 
