@@ -47,6 +47,9 @@ var ErrUpToDate = errors.New("up to date")
 var ErrIsAhead = errors.New("current fast forward from a to b. a is ahead of b already")
 var ErrIsBehind = errors.New("cannot reverse from b to a. b is a is behind a already")
 
+var ErrUnresolvedConflicts = errors.New("merge has unresolved conflicts")
+var ErrMergeActive = errors.New("merging is not possible because you have not committed an active merge")
+
 type ErrClientOutOfDate struct {
 	RepoVer   FeatureVersion
 	ClientVer FeatureVersion
