@@ -15,6 +15,7 @@
 package doltdb
 
 import (
+	"bytes"
 	"context"
 	"errors"
 	"fmt"
@@ -1189,3 +1190,37 @@ func validateTagUniqueness(ctx context.Context, root *RootValue, tableName strin
 
 	return nil
 }
+
+// DebugString returns a human readable string with the contents of this root. If |transitive| is true, row data from
+// all tables is also included. This method is very expensive for large root values, so |transitive| should only be used
+// when debugging tests.
+func (root *RootValue) DebugString(ctx context.Context, transitive bool) string {
+	var buf bytes.Buffer
+	err := types.WriteEncodedValue(ctx, &buf, root.valueSt)
+	if err != nil {
+		panic(err)
+	}
+
+	if transitive {
+		buf.WriteString("\nTables:")
+		root.IterTables(ctx, func(name string, table *Table, sch schema.Schema) (stop bool, err error) {
+			buf.WriteString("\nName:")
+			buf.WriteString(name)
+			buf.WriteString("\n")
+
+			buf.WriteString("Data:\n")
+			data, err := table.GetRowData(ctx)
+			if err != nil {
+				panic(err)
+			}
+
+			err = types.WriteEncodedValue(ctx, &buf, data)
+			if err != nil {
+				panic(err)
+			}
+			return false, nil
+		})
+	}
+
+	return buf.String()
+}
