@@ -34,16 +34,13 @@ import (
 	"github.com/dolthub/dolt/go/store/types"
 )
 
-var fk_dEnv *env.DoltEnv
-var fk_initialRoot *doltdb.RootValue
-
-func init() {
-	fk_dEnv = dtestutils.CreateTestEnv()
-	root, err := fk_dEnv.WorkingRoot(context.Background())
+func setupEditorFkTest(t *testing.T) (*env.DoltEnv, *doltdb.RootValue) {
+	dEnv := dtestutils.CreateTestEnv()
+	root, err := dEnv.WorkingRoot(context.Background())
 	if err != nil {
 		panic(err)
 	}
-	fk_initialRoot, err = ExecuteSql(fk_dEnv, root, `
+	initialRoot, err := ExecuteSql(dEnv, root, `
 CREATE TABLE one (
   pk BIGINT PRIMARY KEY,
   v1 BIGINT,
@@ -74,18 +71,12 @@ CREATE TABLE child (
   v2 BIGINT
 );
 `)
-	if err != nil {
-		panic(err)
-	}
+	require.NoError(t, err)
+
+	return dEnv, initialRoot
 }
 
 func TestTableEditorForeignKeyCascade(t *testing.T) {
-	testRoot, err := ExecuteSql(fk_dEnv, fk_initialRoot, `
-ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE CASCADE ON UPDATE CASCADE;
-ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASCADE ON UPDATE CASCADE;
-`)
-	require.NoError(t, err)
-
 	tests := []struct {
 		name          string
 		sqlStatement  string
@@ -163,10 +154,18 @@ ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASC
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			dEnv, initialRoot := setupEditorFkTest(t)
+
+			testRoot, err := ExecuteSql(dEnv, initialRoot, `
+ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE CASCADE ON UPDATE CASCADE;
+ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASCADE ON UPDATE CASCADE;
+`)
+			require.NoError(t, err)
+
 			root := testRoot
 			for _, sqlStatement := range strings.Split(test.sqlStatement, ";") {
 				var err error
-				root, err = executeModify(context.Background(), fk_dEnv, root, sqlStatement)
+				root, err = executeModify(context.Background(), dEnv, root, sqlStatement)
 				require.NoError(t, err)
 			}
 
@@ -178,10 +177,6 @@ ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASC
 }
 
 func TestTableEditorForeignKeySetNull(t *testing.T) {
-	testRoot, err := ExecuteSql(fk_dEnv, fk_initialRoot, `
-ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE SET NULL ON UPDATE SET NULL;`)
-	require.NoError(t, err)
-
 	tests := []struct {
 		sqlStatement string
 		expectedOne  []sql.Row
@@ -208,10 +203,16 @@ ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE SET NULL ON UP
 
 	for _, test := range tests {
 		t.Run(test.sqlStatement, func(t *testing.T) {
+			dEnv, initialRoot := setupEditorFkTest(t)
+
+			testRoot, err := ExecuteSql(dEnv, initialRoot, `
+ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE SET NULL ON UPDATE SET NULL;`)
+			require.NoError(t, err)
+
 			root := testRoot
 			for _, sqlStatement := range strings.Split(test.sqlStatement, ";") {
 				var err error
-				root, err = executeModify(context.Background(), fk_dEnv, root, sqlStatement)
+				root, err = executeModify(context.Background(), dEnv, root, sqlStatement)
 				require.NoError(t, err)
 			}
 
@@ -232,12 +233,6 @@ func TestTableEditorForeignKeyRestrict(t *testing.T) {
 		"",
 	} {
 		t.Run(referenceOption, func(t *testing.T) {
-			testRoot, err := ExecuteSql(fk_dEnv, fk_initialRoot, fmt.Sprintf(`
-			ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) %s;
-			INSERT INTO one VALUES (1, 1, 1), (2, 2, 2), (3, 3, 3);
-			INSERT INTO two VALUES (1, 1, 1), (2, 2, 2), (3, 3, 3);`, referenceOption))
-			require.NoError(t, err)
-
 			tests := []struct {
 				setup       string
 				trigger     string
@@ -290,17 +285,25 @@ func TestTableEditorForeignKeyRestrict(t *testing.T) {
 
 			for _, test := range tests {
 				t.Run(test.setup+test.trigger, func(t *testing.T) {
+					dEnv, initialRoot := setupEditorFkTest(t)
+
+					testRoot, err := ExecuteSql(dEnv, initialRoot, fmt.Sprintf(`
+			ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) %s;
+			INSERT INTO one VALUES (1, 1, 1), (2, 2, 2), (3, 3, 3);
+			INSERT INTO two VALUES (1, 1, 1), (2, 2, 2), (3, 3, 3);`, referenceOption))
+					require.NoError(t, err)
+
 					root := testRoot
 					for _, sqlStatement := range strings.Split(test.setup, ";") {
 						var err error
-						root, err = executeModify(context.Background(), fk_dEnv, root, sqlStatement)
+						root, err = executeModify(context.Background(), dEnv, root, sqlStatement)
 						require.NoError(t, err)
 					}
 					if test.expectedErr {
-						root, err = executeModify(context.Background(), fk_dEnv, root, test.trigger)
+						root, err = executeModify(context.Background(), dEnv, root, test.trigger)
 						assert.Error(t, err)
 					} else {
-						root, err = executeModify(context.Background(), fk_dEnv, root, test.trigger)
+						root, err = executeModify(context.Background(), dEnv, root, test.trigger)
 						assert.NoError(t, err)
 					}
 				})
@@ -310,12 +313,6 @@ func TestTableEditorForeignKeyRestrict(t *testing.T) {
 }
 
 func TestTableEditorForeignKeyViolations(t *testing.T) {
-	testRoot, err := ExecuteSql(fk_dEnv, fk_initialRoot, `
-ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE CASCADE ON UPDATE CASCADE;
-ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASCADE ON UPDATE CASCADE;
-`)
-	require.NoError(t, err)
-
 	tests := []struct {
 		setup   string
 		trigger string
@@ -359,21 +356,31 @@ ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASC
 
 	for _, test := range tests {
 		t.Run(test.setup+test.trigger, func(t *testing.T) {
+			dEnv, initialRoot := setupEditorFkTest(t)
+
+			testRoot, err := ExecuteSql(dEnv, initialRoot, `
+ALTER TABLE two ADD FOREIGN KEY (v1) REFERENCES one(v1) ON DELETE CASCADE ON UPDATE CASCADE;
+ALTER TABLE three ADD FOREIGN KEY (v1, v2) REFERENCES two(v1, v2) ON DELETE CASCADE ON UPDATE CASCADE;
+`)
+			require.NoError(t, err)
+
 			root := testRoot
 			for _, sqlStatement := range strings.Split(test.setup, ";") {
 				var err error
-				root, err = executeModify(context.Background(), fk_dEnv, root, sqlStatement)
+				root, err = executeModify(context.Background(), dEnv, root, sqlStatement)
 				require.NoError(t, err)
 			}
-			root, err = executeModify(context.Background(), fk_dEnv, root, test.trigger)
+			root, err = executeModify(context.Background(), dEnv, root, test.trigger)
 			assert.Error(t, err)
 		})
 	}
 }
 
 func TestTableEditorSelfReferentialForeignKeyRestrict(t *testing.T) {
+	dEnv, initialRoot := setupEditorFkTest(t)
+
 	ctx := context.Background()
-	root := fk_initialRoot
+	root := initialRoot
 
 	sequentialTests := []struct {
 		statement   string
@@ -428,7 +435,7 @@ func TestTableEditorSelfReferentialForeignKeyRestrict(t *testing.T) {
 	}
 
 	for _, test := range sequentialTests {
-		newRoot, err := executeModify(ctx, fk_dEnv, root, test.statement)
+		newRoot, err := executeModify(ctx, dEnv, root, test.statement)
 		if test.expectedErr {
 			require.Error(t, err)
 			continue
@@ -440,8 +447,10 @@ func TestTableEditorSelfReferentialForeignKeyRestrict(t *testing.T) {
 }
 
 func TestTableEditorSelfReferentialForeignKeyCascade(t *testing.T) {
+	dEnv, initialRoot := setupEditorFkTest(t)
+
 	ctx := context.Background()
-	root := fk_initialRoot
+	root := initialRoot
 
 	sequentialTests := []struct {
 		statement   string
@@ -526,7 +535,7 @@ func TestTableEditorSelfReferentialForeignKeyCascade(t *testing.T) {
 	}
 
 	for _, test := range sequentialTests {
-		newRoot, err := executeModify(ctx, fk_dEnv, root, test.statement)
+		newRoot, err := executeModify(ctx, dEnv, root, test.statement)
 		if test.expectedErr {
 			require.Error(t, err)
 			continue
@@ -538,8 +547,10 @@ func TestTableEditorSelfReferentialForeignKeyCascade(t *testing.T) {
 }
 
 func TestTableEditorSelfReferentialForeignKeySetNull(t *testing.T) {
+	dEnv, initialRoot := setupEditorFkTest(t)
+
 	ctx := context.Background()
-	root := fk_initialRoot
+	root := initialRoot
 
 	sequentialTests := []struct {
 		statement   string
@@ -624,7 +635,7 @@ func TestTableEditorSelfReferentialForeignKeySetNull(t *testing.T) {
 	}
 
 	for _, test := range sequentialTests {
-		newRoot, err := executeModify(ctx, fk_dEnv, root, test.statement)
+		newRoot, err := executeModify(ctx, dEnv, root, test.statement)
 		if test.expectedErr {
 			require.Error(t, err)
 			continue
